/**
 * *
 * The Example is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 *
 * The Example is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Neuroph. If not, see <http://www.gnu.org/licenses/>.
 */
package bollingerbands;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Iterator;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.neuroph.core.NeuralNetwork;
import org.neuroph.core.learning.DataSet;
import org.neuroph.nnet.MultiLayerPerceptron;
import org.neuroph.nnet.learning.BackPropagation;

/**
 *
 * @author Dr.V.Steinhauer
 */
public class BollingerAnalyze {

    private static final Logger logger = Logger.getLogger(BollingerAnalyze.class);
    int inputCount = 5;
    int outputCount = 3;
    ArrayList aDax = new ArrayList();
    static BollingerAnalyze bollingerAnalyze;
    int period = 21; // how many points will be used in Bollinger Bands   

    public BollingerAnalyze() {
        PropertyConfigurator.configure("log4j.properties");
        System.out.println("-------------Build the Net Work-------------");
        buildNetwork();
        System.out.println("-------------Test the Net Work-------------");
        testNetwork();
        System.out.println("-------------Test getRecommendation-------------");
        int r = getRecommendation();//1- predicted UP, 0- didn't predict, -1 - predicted DOWN
    }

    private void buildNetwork() {

        readHistoricData("./test/HistoricData_20110510.csv");
          
        DataSet trainingSet = new DataSet(inputCount + 2, 1);

        double[] daxes = new double[aDax.size()];
        Iterator it = aDax.iterator();
        int ii = -1;
        while (it.hasNext()) {
            daxes[++ii] = (Double) it.next();
        }

        int count = 0;
        int scount = 0;

        double[] d = daxes.clone();
        double[] bu = new double[d.length];
        double[] bw = new double[d.length];
        for (int k = 0; k < d.length; k++) {
            double mittelwert = gleitendesMittel(d, k);
            double sigma = standardAbweichung(d, k);
            bu[k] = mittelwert + 2.0D * sigma;
            bw[k] = mittelwert - 2.0D * sigma;            
        }
        for (int k = 0; k < d.length; k++) {
            boolean bmax = true;
            boolean bmin = true;
            int i0 = k - inputCount;
            if (i0 < 0) {
                i0 = 0;
                continue;
            }
            int i1 = k + outputCount;
            if (i1 > d.length - 1) {
                i1 = d.length - 1;
                continue;
            }
            for (int i = i0; i < i1; i++) {
                if (i == k) {
                    continue;
                }
                if (d[k] < d[i]) {
                    bmax = false;
                }
                if (d[k] > d[i]) {
                    bmin = false;
                }
            }

            double[] line = new double[inputCount + 2];
            double[] out = new double[1];
            if (bmin || bmax) {
                ++scount;
                out[0] = 0.0D;
                if ((d[k] > bu[k] && bmax) || (d[k] < bw[k] && bmin)) {
                    ++count;
                    for (int m = 0; m < inputCount; m++) {
                        line[m] = d[k - inputCount + m];
                    }
                    line[inputCount] = d[k];
                    line[inputCount + 1] = d[k + 1];
                    double[] t = normalize(line);
                    out[0] = 1.0D;
                    trainingSet.addRow(t, out);
                    //test
                    System.out.print("\n");
                    for (int n = 0; n < t.length; n++) {
                        System.out.printf("%6.5f ", t[n]);
                    }
                }
            } else {
                for (int m = 0; m < inputCount; m++) {
                    line[m] = d[k - inputCount + m];
                }
                line[inputCount] = d[k];
                double[] t = normalize(line);
                out[0] = 0.0D;
                trainingSet.addRow(t, out);
            }
        }

        NeuralNetwork neuralNet = new MultiLayerPerceptron(inputCount + 2, inputCount * 2 + 1, 1);
        neuralNet.setLearningRule(new BackPropagation());
        neuralNet.learn(trainingSet);
        neuralNet.save("bollinger.nnet");

        System.out.println("\nextremums number = " + scount + "  bollingers extremums number. = " + count + " full number of points = " + d.length);
        String stcount = Integer.toString(count);
        String snumber = Integer.toString(d.length);
        double p = Double.parseDouble(stcount) / Double.parseDouble(snumber);
        double pd = 100.0D * p;
        System.out.printf("possibility= %5.4f p=%2.1f inputCount=%d %d-th value after last input value is output\n", p, pd, inputCount, outputCount);

        aDax.clear();

    }

    private int getRecommendation() {

        readHistoricData("./test/HistoricData_20110510.csv");
        
        int result = 0; //1- UP, 0- No PREDICTION, -1 - DOWN

        NeuralNetwork neuralNet = NeuralNetwork.load("bollinger.nnet");
        double[] daxes = new double[aDax.size()];
        Iterator it = aDax.iterator();
        int ii = -1;
        while (it.hasNext()) {
            daxes[++ii] = (Double) it.next();
        }

        int count = 0;

        double[] d = daxes.clone();
        double[] bu = new double[d.length];//the bollinger upline
        double[] bw = new double[d.length];//the  bollinger downline
        for (int k = 0; k < d.length; k++) {
            double mittelwert = gleitendesMittel(d, k);
            double sigma = standardAbweichung(d, k);
            bu[k] = mittelwert + 2.0D * sigma;
            bw[k] = mittelwert - 2.0D * sigma;
        }
        for (int k = 0; k < d.length; k++) {
            boolean dmax = true;// is it the  max?
            boolean dmin = true;// is it the  min?
            int i0 = k - inputCount;
            if (i0 < 0) {
                i0 = 0;
                continue;
            }
            int i1 = k + outputCount;
            if (i1 > d.length - 1) {
                i1 = d.length - 1;
                continue;
            }
            for (int i = i0; i < i1; i++) {
                if (i == k) {
                    continue;
                }
                if (d[k] < d[i]) {
                    dmax = false;
                }
                if (d[k] > d[i]) {
                    dmin = false;
                }
            }

            double[] line = new double[inputCount + 2];
            if ((d[k] > bu[k] && dmax) || (d[k] < bw[k] && dmin)) {// sind die bollinger max/min?
                ++count;
                for (int m = 0; m < inputCount; m++) {
                    line[m] = d[k - inputCount + m];
                }
                line[inputCount] = d[k];
                line[inputCount + 1] = d[k + 1];
                // is this line the last values?
                //if(last){
                double[] t = normalize(line);
                double pred = getPredicted(t, neuralNet);
                if (Math.abs(pred - 1.0D) < 0.47) {
                    if (dmax) {
                        result = -1;
                        System.out.print("\n (bollinger method) predicted DOWN! in 15 minutes: " + pred);
                    }
                    if (dmin) {
                        result = 1;
                        System.out.print("\n (bollinger method) predicted UP! in 15 minutes: " + pred);
                    }
                }
                //} the end of the if(last)
            }
        }
        aDax.clear();
        return result;
    }

    private void testNetwork() {

        readHistoricData("./test/HistoricData_20110510.csv");

        NeuralNetwork neuralNet = NeuralNetwork.load("bollinger.nnet");
        int defects = 0;
        double[] daxes = new double[aDax.size()];
        Iterator it = aDax.iterator();
        int ii = -1;
        while (it.hasNext()) {
            daxes[++ii] = (Double) it.next();
        }

        int count = 0;
        int scount = 0;

        double[] d = daxes.clone();
        double[] bu = new double[d.length];
        double[] bw = new double[d.length];
        for (int k = 0; k < d.length; k++) {
            double mittelwert = gleitendesMittel(d, k);
            double sigma = standardAbweichung(d, k);
            bu[k] = mittelwert + 2.0D * sigma;
            bw[k] = mittelwert - 2.0D * sigma;            
        }
        for (int k = 0; k < d.length; k++) {
            boolean bmax = true;
            boolean bmin = true;
            int i0 = k - inputCount;
            if (i0 < 0) {
                i0 = 0;
                continue;
            }
            int i1 = k + outputCount;
            if (i1 > d.length - 1) {
                i1 = d.length - 1;
                continue;
            }
            for (int i = i0; i < i1; i++) {
                if (i == k) {
                    continue;
                }
                if (d[k] < d[i]) {
                    bmax = false;
                }
                if (d[k] > d[i]) {
                    bmin = false;
                }
            }

            double[] line = new double[inputCount + 2];
            double[] out = new double[1];
            if (bmin || bmax) {
                ++scount;
                out[0] = 0.0D;
                if ((d[k] > bu[k] && bmax) || (d[k] < bw[k] && bmin)) {
                    ++count;
                    for (int m = 0; m < inputCount; m++) {
                        line[m] = d[k - inputCount + m];
                    }
                    line[inputCount] = d[k];
                    line[inputCount + 1] = d[k + 1];
                    double[] t = normalize(line);
                    out[0] = 1.0D;
                    double pred = getPredicted(t, neuralNet);
                    if (Math.abs(pred - 1.0D) > 0.47) {
                        defects = defects + 1;                        
                    } else {
                        //System.out.print("\n 1.0 -> " + pred);
                    }
                }
            } else {
                for (int m = 0; m < inputCount; m++) {
                    line[m] = d[k - inputCount + m];
                }
                line[inputCount] = d[k];
                double[] t = normalize(line);
                out[0] = 0.0D;
                double pred = getPredicted(t, neuralNet);
                if (pred > 0.47) {
                    defects = defects + 1;
                    System.out.print("\n defect ----------------------- 0.0 -> " + pred);
                } else {
                    //System.out.print("\n 0.0 -> " + pred);
                }
            }
        }

        System.out.println("\nextremums number = " + scount + "  bollingers extremums number. = " + count + " full number of points = " + d.length + " defects = " + defects);
        String stcount = Integer.toString(count);
        String snumber = Integer.toString(d.length);
        double p = Double.parseDouble(stcount) / Double.parseDouble(snumber);
        double pd = 100.0D * p;
        System.out.printf("possibility= %5.4f p=%2.1f inputCount=%d, %d-th value after last input value is output\n", p, pd, inputCount, outputCount);

        aDax.clear();

    }

    private double getPredicted(double[] t, NeuralNetwork neuralNet) {
        double pred = -1.0D;
        neuralNet.setInput(t);
        neuralNet.calculate();
        double[] networkOutput = neuralNet.getOutput();
        pred = networkOutput[0];
        return pred;
    }

    private double[] normalize(double[] data) {
        double dmin = 99999999999.0D;
        double dmax = -9999999999.0D;
        double epsilon = 1.01;
        for (int i = 0; i < data.length; i++) {
            if (data[i] > dmax) {
                dmax = data[i];
            }
            if (data[i] < dmin) {
                dmin = data[i];
            }
        }
        dmax = dmax * epsilon;
        dmin = dmin / epsilon;
        double[] r = new double[data.length];
        for (int i = 0; i < data.length; i++) {
            r[i] = (data[i] - dmin) / (dmax - dmin);
        }
        return r;
    }

    private void readHistoricData(String fileName) {
        File file = new File(fileName);
        logger.info("file = " + fileName + ". It will be read data");
        try {
            FileInputStream fis = new FileInputStream(file);
            BufferedReader dis = new BufferedReader(new InputStreamReader(fis));
            String s;
            while ((s = dis.readLine()) != null) {
                try {
                    s = s.replaceAll(" ", ";");
                    String[] s1 = s.split(";");                    
                    String s01 = s1[2].trim();
                    s01 = s01.replace('\"', ' ');
                    s01 = s01.replace('\"', ' ');
                    if (s01.indexOf(",") > 0) {
                        s01 = s01.replaceAll("\\.", "");
                        s01 = s01.replaceAll(",", ".");
                        s01 = s01.replace('\"', ' ');
                        System.out.println(s01);
                    }
                    double dax = Double.parseDouble(s01);
                    aDax.add(dax);
                } catch (NumberFormatException ne) {
                    logger.error(ne);
                }
            }
            fis.close();
        } catch (IOException ioe) {
            logger.error(ioe);
        }
    }

    private double gleitendesMittel(double[] data, int i) {
        double gm = 0.0D;
        if (i <= 1) {
            return 0.0D;
        }
        int n0 = i - period;
        if (n0 < 0) {
            n0 = 0;
        }
        int counter = 0;
        for (int k = n0; k <= i; k++) {
            ++counter;
            gm = gm + data[k];

        }
        gm = gm / counter;
        return gm;
    }

    private double standardAbweichung(double[] data, int i) {

        double gm = 0.0D;
        if (i <= 1) {
            return 0.0D;
        }
        int n0 = i - period;
        if (n0 < 0) {
            n0 = 0;
        }
        int counter = 0;
        for (int k = n0; k <= i; k++) {
            ++counter;
            gm = gm + data[k];

        }
        gm = gm / counter;

        double sum = 0.0D;
        for (int k = n0; k <= i; k++) {
            double v = data[k];
            sum = sum + (v - gm) * (v - gm);
        }
        gm = Math.sqrt(sum / (counter - 1));
        return gm;
    }

    public static void main(String[] args) {
        bollingerAnalyze = new BollingerAnalyze();
    }
}
