/**
 * *
 * The Example is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 *
 * The Example is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Neuroph. If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * To filter the training sets
 */
package stockmarketcommittee.timeseries;

import java.util.Iterator;
import java.util.List;
import org.neuroph.core.NeuralNetwork;
import org.neuroph.core.learning.DataSet;
import org.neuroph.core.learning.DataSetRow;
import org.neuroph.nnet.MultiLayerPerceptron;
import org.neuroph.nnet.learning.LMS;

/**
 *
 * @author vs
 */
public class Filter {

    DataSet trainingSet;

    public Filter(DataSet trainingSet) {
        this.trainingSet = trainingSet;
    }

    public DataSet doFilterOneRow() {
        DataSet trainingSetBest = trainingSet;
        NeuralNetwork neuralNet = new MultiLayerPerceptron(4, 9, 1);      
        neuralNet.learn(trainingSet);
        double Rfac = rfacCalculate(neuralNet);
        System.out.println("------------ reducing of dataset with one row ------------");
        System.out.println("Start value of Rfac:" + Rfac);
        for (int i = 0; i < trainingSet.size(); i++) {
            DataSet trainingTestSet = trainingSet;
            trainingTestSet.removeRowAt(i);
            NeuralNetwork testNN = new MultiLayerPerceptron(4, 9, 1);           
            testNN.learn(trainingTestSet);
            double r = rfacCalculate(testNN);
            if (r < Rfac) {
                Rfac = r;
                trainingSetBest = trainingTestSet;
                System.out.println("Rfac:" + Rfac + " try to the row i:" + i + " r:" + r);
            }
        }
        return trainingSetBest;
    }

    /**
     * Calculation of Rfactor with full trainingSet and difined network
     *
     * @param neuralNet
     * @return
     */
    public double rfacCalculate(NeuralNetwork neuralNet) {
        double sumA = 0.0D;
        double sumB = 0.0D;
        List<DataSetRow> fullList = trainingSet.getRows();
        Iterator it = fullList.iterator();
        while (it.hasNext()) {
            DataSetRow dsr = (DataSetRow) it.next();
            double[] input = dsr.getInput();
            neuralNet.calculate();
            double[] networkOutput = neuralNet.getOutput();
            double[] desiredOutput = dsr.getDesiredOutput();
            sumA = sumA + Math.abs(networkOutput[0] - desiredOutput[0]);
            sumB = sumB + desiredOutput[0];
        }
        return sumA / sumB;
    }
}
