/**
 * *
 * The Example is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 *
 * The Example is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Neuroph. If not, see <http://www.gnu.org/licenses/>.
 */
package stockmarketcommittee.timeseries;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import org.neuroph.core.learning.DataSet;

public class TrainingSetGetter {

    private DataSet trainingSet;
    private double maxvalue = 1.0D;
    private double minvalue = 0.0D;
    private List dataList;
    private List normalizedDataList;
    private double expander = 1.00D;

    /**
     *
     * @param i -> 0 - DAX, 1 - other
     */
    public TrainingSetGetter(int i) {
        System.out.println("------>" + i);
        if (i == 0) {
            setDaxList();
            System.out.println("0");
        }
        if (i == 1) {
            setOtherList();
            System.out.println("1");
        }
        maxvalue = getMaxvalueX(dataList);
        minvalue = getMinvalueX(dataList);
    }

    public void setMaxvalue(double maxvalue) {
        this.maxvalue = maxvalue;
    }

    public void setMinvalue(double minvalue) {
        this.minvalue = minvalue;
    }

    public void normalize() {
        normalizedDataList = new ArrayList();
        Iterator it = dataList.iterator();
        while (it.hasNext()) {
            double v = (Double) it.next();
            normalizedDataList.add(0.8 * (v - minvalue) / (maxvalue - minvalue) + 0.1);         
        }
    }

    /**
     *
     * @param n number of input neurons
     * @param m number of output neurons
     * @return dataset for training
     */
    public DataSet getTrainingSet(int n, int m) {
        trainingSet = new DataSet(n, m);
        for (int k = 0; k < normalizedDataList.size() - n - m + 2; k = k + 1) {
            System.out.println("\n");
            double[] in = new double[n];
            for (int i = 0; i < n; i++) {
                in[i] = (Double) normalizedDataList.get(k + i);
                //System.out.print(" " + in[i]);
                double test = (in[i] - 0.1) * (maxvalue - minvalue) / 0.8 + minvalue;
                System.out.print(" " + test);
            }
            try {
                double[] out = new double[m];
                for (int i = 0; i < m; i++) {
                    out[i] = (Double) normalizedDataList.get(k + n + i);                    
                    double test = (out[i] - 0.1) * (maxvalue - minvalue) / 0.8 + minvalue;
                    System.out.print(" " + test);
                }
                trainingSet.addRow(in, out);
            } catch (Exception e) {
                System.out.println("\n");
                break;
            }
        }
        return trainingSet;
    }

    public List getTestLine(int n) {
        List list = new ArrayList();
        int counter = normalizedDataList.size() - n;
        for (int i = counter; i < normalizedDataList.size(); i++) {
            list.add((Double) normalizedDataList.get(i));           
        }
        return list;
    }

    private void setDaxList() {
        expander = 1.30D;
        dataList = new ArrayList();
        dataList.add(3710.0D);
        dataList.add(3690.0D);
        dataList.add(3890.0D);
        dataList.add(3695.0D);
        dataList.add(3666.0D);
        dataList.add(3692.0D);
        dataList.add(3886.0D);
        dataList.add(3914.0D);
        dataList.add(3956.0D);
        dataList.add(3953.0D);
        dataList.add(4044.0D);
        dataList.add(3987.0D);
        dataList.add(3996.0D);
        dataList.add(4043.0D);
        dataList.add(4068.0D);
        dataList.add(4176.0D);
        dataList.add(4187.0D);
        dataList.add(4223.0D);
        dataList.add(4259.0D);
        dataList.add(4203.0D);
        dataList.add(3989.0D);

    }

    private void setOtherList() {
        expander = 1.08D;
        dataList = new ArrayList();
        dataList.add(8.41D);
        dataList.add(8.45D);
        dataList.add(8.61D);
        dataList.add(8.70D);
        dataList.add(8.65D);
        dataList.add(8.56D);
        dataList.add(8.51D);
        dataList.add(8.77D);
        dataList.add(8.75D);
        dataList.add(8.77D);
        dataList.add(8.86D);
        dataList.add(8.85D);
        dataList.add(9.17D);
        dataList.add(9.15D);
        dataList.add(9.15D);
        dataList.add(9.15D);
        dataList.add(9.15D);
        dataList.add(9.15D);
        dataList.add(9.21D);
        dataList.add(9.25D);
        dataList.add(9.25D);
        dataList.add(9.25D);
        dataList.add(9.35D);
        dataList.add(9.46D);
        dataList.add(9.55D);
        dataList.add(9.51D);
        dataList.add(9.69D);
        dataList.add(9.81D);
        dataList.add(10.6D);

    }

    private List clone(List ds) {
        List list = new ArrayList();
        Iterator it = ds.iterator();
        while (it.hasNext()) {
            list.add(it.next());
        }
        return list;
    }

    private double getMaxvalueX(List ds) {
        List dsX = clone(ds);
        Collections.sort(dsX);
        System.out.println("Max.value is " + dsX.get(ds.size() - 1));
        return (Double) dsX.get(ds.size() - 1) * expander;
    }

    private double getMinvalueX(List ds) {
        List dsX = clone(ds);
        Collections.sort(dsX);
        System.out.println("Min.value is " + dsX.get(0));
        return (Double) dsX.get(0) / expander;
    }

    public double getMaxvalue() {
        return maxvalue;
    }

    public double getMinvalue() {
        return minvalue;
    }

}
